@@ -18,6 +18,7 @@ try:
     import html2text
 except ImportError:
     html2text = None
+from sql import Literal
 from sql.conditionals import Coalesce
 from sql.functions import CurrentTimestamp
 from sql.operators import Equal
@@ -35,8 +36,6 @@ from trytond.sendmail import sendmail_transactional
 from trytond.ir.session import token_hex
 from trytond.res.user import LoginAttempt, CRYPT_CONTEXT
 
-__all__ = ['User', 'UserAuthenticateAttempt', 'UserSession',
-    'EmailValidation', 'EmailResetPassword']
 logger = logging.getLogger(__name__)
 
 
@@ -83,7 +82,7 @@ class User(DeactivableMixin, ModelSQL, ModelView):
     reset_password_token = fields.Char('Reset Password Token', select=True)
     reset_password_token_expire = fields.Timestamp(
         'Reset Password Token Expire')
-    party = fields.Many2One('party.party', 'Party')
+    party = fields.Many2One('party.party', 'Party', ondelete='RESTRICT')
 
     @classmethod
     def __setup__(cls):
@@ -92,7 +91,7 @@ class User(DeactivableMixin, ModelSQL, ModelView):
         cls._sql_constraints += [
             ('email_exclude',
                 Exclude(table, (table.email, Equal),
-                    where=table.active == True),
+                    where=table.active == Literal(True)),
                 'web_user.msg_user_email_unique'),
             ]
         cls._buttons.update({
@@ -208,7 +207,7 @@ class User(DeactivableMixin, ModelSQL, ModelView):
     @classmethod
     def check_password(cls, password, hash_):
         if not hash_:
-            return False, None
+            return False
         try:
             return CRYPT_CONTEXT.verify_and_update(password, hash_)
         except ValueError:
@@ -361,7 +360,8 @@ class User(DeactivableMixin, ModelSQL, ModelView):
         email = email.lower()
 
         # Prevent brute force attack
-        Transaction().atexit(time.sleep, 2 ** Attempt.count(email) - 1)
+        Transaction().atexit(
+            time.sleep, random.randint(0, 2 ** Attempt.count(email) - 1))
 
         users = cls.search([
                 ('email', '=', email),
